@@ -1,27 +1,7 @@
-.confint.boot <- function(boot_args, object, expr_t, t_fn=NULL, parm, boot.ci, level, ...) {
-  spaMM_boot_args <- intersect(names(boot_args),names(formals(spaMM_boot)))
-  spaMM_boot_args <- boot_args[spaMM_boot_args]
-  spaMM_boot_args$object <- object
-  if (is.null(spaMM_boot_args$type)) spaMM_boot_args$type <- "residual"
-  boot.ci_args <- intersect(names(boot_args),names(formals(boot.ci)))
-  boot.ci_args <- boot_args[boot.ci_args]
-  if (is.null(t_fn)) {
-    spaMM_boot_args$simuland <- function(y, ...) {
-      upd <- update_resp(object, newresp=y)
-      eval(expr_t, list(hlfit=upd))
-    }
-    boot.ci_args$t0 <- eval(expr_t, list(hlfit=object))
-  } else {
-    spaMM_boot_args$simuland <- function(y, ...) {
-      upd <- update_resp(object, newresp=y)
-      t_fn(upd)
-    }
-    boot.ci_args$t0 <- t_fn(object, ...)
-  }
-  ts <- drop(do.call(spaMM_boot,spaMM_boot_args, ...)[["bootreps"]])
+.boot_single_par <- function(boot.ci_args, t0, ts, verbose) {
+  boot.ci_args$t0 <- t0
   boot.ci_args$t <- ts
   boot.ci_args$boot.out <- list(R = length(ts), sim="parametric")
-  boot.ci_args$conf <- level
   if (is.null(boot.ci_args$ci_type)) {
     boot.ci_args$type <- c("basic","perc","norm")
   } else {
@@ -29,11 +9,57 @@
     boot.ci_args$ci_type <- NULL
   }
   resu <- do.call("boot.ci", boot.ci_args)
-  resu$call <- match.call()
+  ## resu$call is shown and this may be ugly bc of the long t vector. print.bootci() uses dput(), which has no generic for that. We will wrap the 
+  ## print.bootci() call in a print.bootci4call() that locally alterns the $call for nicer printing.
+  class(resu) <- c("bootci4print", class(resu))
+  if (verbose) print(resu)
+  resu
+}
+
+
+.confint_boot <- function(boot_args, object, expr_t, t_fn=NULL, parm, boot.ci, level, verbose, ...) {
+  spaMM_boot_args <- intersect(names(boot_args),names(formals(spaMM_boot)))
+  spaMM_boot_args <- boot_args[spaMM_boot_args]
+  spaMM_boot_args$object <- object
+  if (is.null(spaMM_boot_args$type)) spaMM_boot_args$type <- "residual"
+  boot.ci_args <- intersect(names(boot_args),names(formals(boot.ci)))
+  boot.ci_args <- boot_args[boot.ci_args]
+  boot.ci_args$conf <- level
+  if (is.null(t_fn)) {
+    spaMM_boot_args$simuland <- function(y, ...) {
+      upd <- update_resp(object, newresp=y)
+      eval(expr_t, list(hlfit=upd))
+    }
+    t0 <- eval(expr_t, list(hlfit=object))
+  } else {
+    spaMM_boot_args$simuland <- function(y, ...) {
+      upd <- update_resp(object, newresp=y)
+      t_fn(upd)
+    }
+    t0 <- t_fn(object, ...)
+  }
+  ts <- drop(do.call(spaMM_boot,spaMM_boot_args, ...)[["bootreps"]])
+  if ((np <- NCOL(ts))>1L) {
+    resu <- vector("list",np)
+    for (colit in seq_len(np)) resu[[colit]] <- .boot_single_par(boot.ci_args, t0=t0[colit], ts=ts[,colit], verbose=verbose)
+    names(resu) <- colnames(ts)
+  } else resu <- .boot_single_par(boot.ci_args, t0=t0, ts=ts, verbose=verbose)
   return(resu)
 }
 
-.confint.LRT <- function(level, parm, object, verbose) {
+.confint_LRT <- function(level, parm, object, verbose) {
+  if ((np <- length(parm))>1L) {
+    resu <- vector("list",np)
+    for (colit in seq_len(np)) resu[[colit]] <- .confint_LRT_single_par(level=level, parm=parm[colit], object=object, verbose=verbose)
+    names(resu) <- parm
+  } else {
+    resu <- .confint_LRT_single_par(level=level, parm=parm, object=object, verbose=verbose)
+  }
+  return(resu)
+}
+
+
+.confint_LRT_single_par <- function(level, parm, object, verbose) {
   dlogL <- qchisq(level,df=1)/2
   znorm <- qnorm((1+level)/2)
   if (is.character(parm)) {
@@ -47,7 +73,8 @@
     attr(parm,"col") <- parmcol
   }
   llc <- getCall(object)
-  lc <- switch(paste(llc[[1L]]),
+  if (is.null(fnname <- object$how$fnname)) fnname <- .get_bare_fnname(llc[[1L]])
+  lc <- switch(fnname,
                "corrHLfit" = get_HLCorcall(object,fixed=llc$ranFix),
                "fitme" = get_HLCorcall(object,fixed=llc$fixed), # HLfit or HLCor call
                "HLCor" = get_HLCorcall(object,fixed=llc$ranPars),
@@ -61,7 +88,7 @@
                 stop(paste("confint does not yet handle HLmethod",paste(HL,collapse=" "),
                            "(or ",c(llc$method,llc$HLmethod),").",sep=" ")))
   beta_cov <- .get_beta_cov_any_version(object)
-  beta_se <- sqrt(diag(x=beta_cov))
+  beta_se <- sqrt(diag(x=beta_cov))[parm]
   asympto_abs_Dparm <- znorm* beta_se
   X.pv <- lc$processed$AUGI0_ZX$X.pv
   X_is_scaled <- ( ! is.null(attr(X.pv,"scaled:scale")))
@@ -69,7 +96,7 @@
   # FIXME test processed nature to prevent multinom stuff here
   intervalinfo <- list(fitlik=object$APHLs[[lik]],
                        targetlik=object$APHLs[[lik]]-dlogL,
-                       parm=parm,
+                       parm=parm, # name, vs $MLparm: ML value
                        asympto_abs_Dparm=asympto_abs_Dparm)
   oldopt <- .options.processed(lc$processed, intervalInfo=intervalinfo, augZXy_cond=FALSE)
   if (X_is_scaled) {
@@ -163,6 +190,7 @@
     init_beta <- object$fixef-asympto_abs_Dparm/fac
     if (X_is_scaled) init_beta <- .scale(beta=init_beta,X=X.pv) ## using locally saved X.pv
     lc$processed$intervalInfo$init <- init_beta[parm]
+    lc$processed$intervalInfo$init_v_h <- object$v_h
     if (! is.null(trTemplate)) {
       anyObjfnCall.args <- as.list(lc[-1L]) ## includes processed, ranPars, controlS.dist, control.HLfit...
       anyObjfnCall.args$skeleton <- trTemplate
@@ -225,17 +253,16 @@
   .options.processed(lc$processed, oldopt)
   interval <- c(lowerfit$fixef[parm],upperfit$fixef[parm])
   names(interval) <- paste(c("lower","upper"),parm)
+  if (verbose) print(interval)
   return(list(lowerfit=lowerfit,upperfit=upperfit,interval=interval))
 }
 
 
 confint.HLfit <- function(object, parm, level=0.95, verbose=TRUE, 
                           boot_args=NULL,...) {
-  if ( .REMLmess(object,return_message=FALSE)) {
-    warning("REML fits are not quite suitable for computing intervals for fixed effects.")
-  } 
+   
   if (is.character(parm)) {
-    if (is.list(boot_args)) expr_t <- substitute(fixef(hlfit)[[parm]], list(parm=parm))
+    if (is.list(boot_args)) expr_t <- substitute(fixef(hlfit)[parm], list(parm=parm))
     t_fn <- NULL
   } else if (is.function(parm)) {
     t_fn <- parm
@@ -245,12 +272,13 @@ confint.HLfit <- function(object, parm, level=0.95, verbose=TRUE,
     #if (is.null(boot_args)) boot_args <- list(nsim=999L) fail is the example from the doc...
   }
   if (  is.list(boot_args)) {
-    boot_res <- .confint.boot(boot_args, object, expr_t, t_fn, parm, boot.ci, level) 
-    if (verbose) print(boot_res)
+    boot_res <- .confint_boot(boot_args, object, expr_t, t_fn, parm, boot.ci, level, verbose = verbose) 
     invisible(boot_res)
   } else {
-    resu <- .confint.LRT(level, parm, object, verbose)
-    if (verbose) print(resu$interval)
+    if ( .REMLmess(object,return_message=FALSE)) {
+      warning("REML fits are not quite suitable for computing intervals for fixed effects.")
+    }
+    resu <- .confint_LRT(level, parm, object, verbose)
     invisible(resu)
   }
 }
