@@ -25,7 +25,7 @@
   return(mc)
 }
 
-.preprocess_fitme <- function(formula,data, ## matches minimal call of HLfit
+.preprocess_fitme <- function(formula,data,
                              family=gaussian(),
                              init=list(),
                              fixed=list(), ## replaces ranFix
@@ -100,15 +100,16 @@ fitme <- function(formula,data, ## matches minimal call of HLfit
   assign("spaMM_glm_conv_crit",list(max=-Inf) , envir=environment(spaMM_glm.fit))
   time1 <- Sys.time()
   oricall <- match.call(expand.dots=TRUE) ## mc including dotlist
-  oricall$formula <- .preprocess_formula(formula) 
+  mc <- oricall
+  mc[[1L]] <- get(".preprocess_formula", asNamespace("spaMM"))  ## https://stackoverflow.com/questions/10022436/do-call-in-combination-with
+  oricall$formula <- mc$formula <- eval(mc,parent.frame()) # 
   ## : among other effects, forces eval of promise for formula, so re-evaluating the call later will work 
   ## [cf probitgem re-evaluating fitme(form,.....) in eval_smoothtest()]
-  mc <- oricall
-  mc[[1L]] <- get(".check_args_fitme", asNamespace("spaMM")) ## https://stackoverflow.com/questions/10022436/do-call-in-combination-with
+  mc[[1L]] <- get(".check_args_fitme", asNamespace("spaMM")) 
+  mc <- eval(mc,parent.frame()) # 
+  mc[[1L]] <- get(".preprocess_fitme", asNamespace("spaMM"))
   mc <- eval(mc,parent.frame()) # returns modified call including an element 'processed'
-  mc[[1L]] <- get(".preprocess_fitme", asNamespace("spaMM")) ## https://stackoverflow.com/questions/10022436/do-call-in-combination-with
-  mc <- eval(mc,parent.frame()) # returns modified call including an element 'processed'
-  mc[[1L]] <- get("fitme_body", asNamespace("spaMM")) ## https://stackoverflow.com/questions/10022436/do-call-in-combination-with
+  mc[[1L]] <- get("fitme_body", asNamespace("spaMM")) 
   hlcor <- eval(mc,parent.frame()) 
   .check_conv_glm_reinit()
   if (inherits(hlcor,"HLfitlist")) {
@@ -121,6 +122,7 @@ fitme <- function(formula,data, ## matches minimal call of HLfit
   lsv <- c("lsv",ls())
   if ( ! .is.multi(family) && ! is.call(hlcor) ) {
     hlcor$how$fit_time <- .timerraw(time1)
+    hlcor$how$fnname <- "fitme"
     hlcor$fit_time <- structure(hlcor$how$fit_time,
                                 message="Please use how(<fit object>)[['fit_time']] to extract this information cleanly.")
   }
