@@ -176,13 +176,19 @@
       if (is.null(names(Zlist))) names(Zlist) <- Znames <- paste(seq_len(length(Zlist)))
       ## logic is Z[nresp,nUniqueRespLoc].A[nUniqueRespLoc,nHiddenv].L[nHiddenv,nHiddenv]
       for (char_rd in Znames) {
-        if ( ! is.null(AMatrices[[char_rd]])) {
+        Amatrix <- AMatrices[[char_rd]]
+        if ( ! is.null(Amatrix)) {
+          is_incid <- attr(Zlist[[char_rd]],"is_incid")
+          if (inherits(Amatrix,"pMatrix")) {
+            Amatrix <- as(Amatrix,"ngTMatrix")
+          } else is_incid <- NULL
           colnams <- colnames(Zlist[[char_rd]])
-          if ( ! setequal(rownames(AMatrices[[char_rd]]),colnams)) {
+          if ( ! setequal(rownames(Amatrix),colnams)) {
             stop(paste0("Any 'A' matrix must have row names that match the levels of the random effects\n",
                         "(i.e. the colnames of the 'Z' design matrix)"))
           }
-          Zlist[[char_rd]] <- Zlist[[char_rd]] %*% AMatrices[[char_rd]][colnams,] # for the IMRF model, Z must be identity
+          Zlist[[char_rd]] <- Zlist[[char_rd]] %*% Amatrix[colnams,] # for the IMRF model, Z must be identity
+          attr(Zlist[[char_rd]],"is_incid") <- is_incid
         }
       }
       ## why this: ?? => removed
@@ -197,6 +203,15 @@
   return(Zlist)
 }
 
+.assign_ZAfix <- function(processed) { # _F I X M E_ extend to all cases of ZAfix <- .ad_hoc_cbind(...) ?
+  ZAfix <- .ad_hoc_cbind(processed$ZAlist, as_matrix=FALSE)   
+  if (processed$is_spprec) {
+    if ( ! inherits(ZAfix,"sparseMatrix")) ZAfix <- as(ZAfix,"dgCMatrix") # .Rcpp_as_dgCMatrix(ZAfix) ## 
+    processed$AUGI0_ZX$is_unitary_ZAfix <- FALSE
+  }
+  processed$AUGI0_ZX$ZAfix <- ZAfix
+}
+
 .evalWrapper <- function(object,
                          element, ## using try() has a visible impact on speed. Hence, the element must exist in the envir.
                          from=NULL) { ## exported for programming purposes
@@ -361,127 +376,6 @@
   return(lFix)
 }
 
-# function called within HLfit for missing inits... 
-# but also sometimes in fitme_body prior to optimization (cf complex condition for call of .eval_init_lambda_guess())
-.get_inits_by_glm <- function(processed, family=processed$family, reset=FALSE) {
-  if (reset || is.null(processed$envir$inits_by_glm)) { 
-    ## if .get_inits_by_glm is called prior to optimization the family parameters may not be assigned, so: 
-    if (family$family=="negbin") {
-      checktheta <- suppressWarnings(try(environment(family$aic)$shape,silent=TRUE))
-      if (inherits(checktheta,"try-error")) family <- Poisson(family$link, trunc=environment(family$aic)$trunc) 
-    } else if (family$family=="COMPoisson") {
-      checknu <- suppressWarnings(try(environment(family$aic)$nu,silent=TRUE))
-      if (inherits(checknu,"try-error")) family <- poisson() ## do not use the loglambda link of the COMPoisson!
-    }
-    y <- processed$y ## requested by the formula
-    if (family$family=="binomial" && NCOL(y)==1L) { 
-      BinomialDen <- processed$BinomialDen ## requested by the formula
-      begform <-"cbind(y,BinomialDen-y)~"  
-    } else {begform <-"y~"}
-    ###################################################if (pforpv==0) {endform <-"0"} else 
-    X.pv <- processed$AUGI0_ZX$X.pv ## possibly requested by the formula
-    pforpv <- ncol(X.pv)
-    if(pforpv) {
-      endform <-"X.pv-1" ## pas besoin de rajouter une constante vue qu'elle est deja dans X
-    } else {
-      if (family$family %in% c("binomial","poisson")) {
-        endform <- "1" ## no meaningful glm without fixed effect in this case !
-      } else {endform <- "0"}
-    }
-    locform <- as.formula(paste(begform, endform))
-    prior.weights <- processed$prior.weights   ## do not try to eval() it outside of the .wfit function call; else nasty crashes may occur.
-    n_lambda <- sum(attr(processed$ZAlist,"Xi_cols"))
-    if (family$family %in% c("Gamma","gaussian")) {lam_fac <- 1/(n_lambda+1L)} else lam_fac <- 1/n_lambda
-    resu <- list() 
-    if (family$family=="gaussian" && family$link=="identity") {
-      if (inherits(X.pv,"sparseMatrix")) {
-        resglm <- .spaMM_lm.wfit(x=X.pv,y=y,offset=processed$off,w=eval(prior.weights))
-      } else resglm <- lm.wfit(x=X.pv,y=y,offset=processed$off,w=eval(prior.weights))
-      fv <- fitted(resglm)
-      dev <- resid(resglm)^2
-      if (! is.null(resglm$weights)) dev <- dev/resglm$weights
-      resu$lambda <- resu$phi_est <- sum(dev)/resglm$df.residual # /lam_fac
-    } else { ## GLM
-      #
-      resglm <- .tryCatch_W_E(glm.fit(x=X.pv, 
-                                      y=processed$HLframes$Y, 
-                                      weights = eval(prior.weights), 
-                                      offset = processed$off, family = family, 
-                                      control = processed[["control.glm"]]))$value 
-      if (inherits(resglm,"error") || 
-          ( ! resglm$converged && any(fitted(resglm)>1e20)) # this occurred in Gamma(log) models or negbin(log)
-          ) {
-        if (TRUE) {
-          #control.glm <- processed[["control.glm"]]
-          #control.glm$maxit <- control.glm$maxit*2
-          resglm <- spaMM_glm.fit(x=X.pv, 
-                        y=processed$HLframes$Y, 
-                        weights = eval(prior.weights), 
-                        offset = processed$off, family = family, 
-                        control = processed[["control.glm"]])
-          # F I X M E but this may still be poor; default control is still maxit=25. Think about (1) better diagnotic than any(fitted(resglm)>1e20) ? (2) check on non-convergence at this second resglm ? 
-        } else {
-          resglm <- withCallingHandlers(
-            {
-              #control.glm <- processed[["control.glm"]]
-              #control.glm$maxit <- control.glm$maxit*2
-              spaMM_glm.fit(x=X.pv, 
-                            y=processed$HLframes$Y, 
-                            weights = eval(prior.weights), 
-                            offset = processed$off, family = family, 
-                            control = processed[["control.glm"]])
-            },
-            warning = function(w){
-              ## Ignore convergence diagnostics from *this* spaMM_glm.fit() call (only, as it is the first call within HLfit)
-              #if(w$message != "glm.fit: algorithm did not converge" ) # the message is never this !
-              warning(w$message)
-              invokeRestart("muffleWarning")
-            } 
-          )
-        }
-        if ( ( ! .spaMM.data$options$spaMM_glm_conv_silent)
-             && (conv_crit <- environment(spaMM_glm.fit)$spaMM_glm_conv_crit$max>0)) {
-          resu$conv_info <- paste(".get_inits_by_glm() -> spaMM_glm.fit did not yet converge at iteration",
-                                  resglm$iter,"(criterion:",
-                                paste(names(conv_crit),"=",signif(unlist(conv_crit),3),collapse=", "),").\n",
-                                "Use <fitting function>(., control.glm=list(maxit=<larger value>,..)) to control this.")
-          assign("spaMM_glm_conv_crit",list(max=-Inf) , envir=environment(spaMM_glm.fit)) # So that no-convergence in these glms will not be warned about
-        }
-      }
-      #
-      resu$phi_est <- as.numeric(deviance(resglm)/resglm$df.residual) #/n_lambda 
-      if (family$family=="binomial" && max(resglm$prior.weights)==1L) { ## binary response
-        resu$lambda <- 1
-      } else {
-        # fv <- fitted(resglm)
-        # resu$lambda <- sum(resid(resglm)^2/(resglm$prior.weights*family$variance(fv)))/resglm$df.residual # was a birth pang...
-        # resid(resglm) gives resglm$residuals, which are the "working residuals" ie (y-mu)/mu.eta one the RHS of the IRLS equation
-        # hence they are not an appropriate way to reconstruct the deviance residuals
-        # which are given by with(resglm, family$dev.resids(y, mu=fitted.values, weights)), already taking prior weights into account
-        # but this leads to 
-        resu$lambda <- resu$phi_est
-      }
-    } ## end else ('GLM')
-    if (pforpv) {
-      ## Two potential problems (1) NA's pour param non estimables (cas normal); 
-      ## (2) "glm.fit: fitted probabilities numerically 0 or 1 occurred" which implies separation or large offset
-      if (max(abs(c(coefficients(resglm))),na.rm=TRUE)>1e10) { ## na.rm v1.2 
-        warning(paste0("(!) Apparent divergence of estimates in a *GLM* analysis of the data.\n",
-                       "    Check your data for extreme values, separation or bad offset values."))
-      } 
-      beta_eta <- c(coefficients(resglm)) ## this may include NA's. Testcase: HLfit(Strength ~ Material*Preheating+Method,data=weld)
-      if (all(names(beta_eta)=="X.pv")) { ## si la formula etait y ~X.pv-1
-        names(beta_eta) <- colnames(resglm$model$X.pv)
-      } else names(beta_eta) <- gsub("^X\\.pv","",names(beta_eta)) ## removes initial "X.pv" without guessing any order or length
-      resu$beta_eta <- beta_eta
-    } 
-    resu$converged <- resglm$converged
-    #parent.env(environment(processed$get_inits_by_glm)) <- environment(stats::glm)
-    processed$envir$inits_by_glm <- resu
-  } 
-  return(processed$envir$inits_by_glm)
-} 
-
 .preprocess_HLmethod <- function(HLmethod, family, lcrandfamfam) {
   ## conversion from user-friendly format to standard 'XX(...)' format
   ## first index is for (0) h / (1) p_v(h) / (2) p^s_v(h) ie whether h lik or marginal lik is used for fixed effect estimation
@@ -578,11 +472,16 @@
   processed$canonicalLink <- .is_link_canonical(family)  
   #
   GLMMbool <- (length(processed$lcrandfamfam) && all(processed$lcrandfamfam=="gaussian") ) ## only allowed gaussian rand.family is gaussian(identity) 
+  const_pw <- ( ! attr(processed$prior.weights,"is_call"))
   if (GLMMbool) {
     LMMbool <- (family$family=="gaussian" && processed$canonicalLink) 
-  } else LMMbool <- FALSE
+    LLM_const_w <- (LMMbool && const_pw) # code note really tested (_F I X M E_)
+  } else LMMbool <- LLM_const_w <- FALSE
   processed$LMMbool <- LMMbool
+  processed$LLM_const_w <- LLM_const_w
   processed$GLMMbool <- GLMMbool
+  processed$GLGLLM_const_w <- (processed$GLMMbool && # const w.ranef
+                                 family$family=="Gamma" && family$link=="log" && const_pw) # cnst w.resid 
   # no return value
 }
 
@@ -901,23 +800,28 @@ as_precision <- function(corrMatrix) {
   return(X.pv)
 }
 
-.preprocess_formula <- function(formula) {
+.preprocess_formula <- function(formula, control.HLfit=NULL, ...) {
   if (inherits(formula,"predictor")) { 
     return(formula) ## happens eg in confint
     # stop("Do not call '.preprocess_formula' on a predictor object.")
   }
+  env <- control.HLfit$formula_env 
+  ## By partial matching, if control.HLfit was absent from the parent call,
+  ## 'control' is matched to the present control.HLfit. Then control$formula_env will be effective
+  ## (interesting feature here as control can hold all the info from everything from the control.[...] args.
+  if (is.null(env)) env <- new.env() # emptyenv() does not work as base functions must be accessible...
   formlen <- length(formula)
   formula[[formlen]] <- .expandDoubleVerts(formula[[formlen]])
   rhs <- .expand_multIMRFs(formula[[formlen]]) 
   hyper_info <- attr(rhs,"hyper_info")
   if ( !is.null(hyper_info)) {
     hyper_form <- formula 
-    environment(hyper_form) <- new.env()
+    environment(hyper_form) <- env
     hyper_info$formula <- hyper_form
     attr(rhs,"hyper_info") <- NULL
   }
   formula[[formlen]] <- rhs
-  environment(formula) <- new.env() # zut <- y~x; rezut <- spaMM:::.preprocess_formula(zut); ls(environment(zut)) confirms that the original 'formula's environment is unaffected
+  environment(formula) <- env # zut <- y~x; rezut <- spaMM:::.preprocess_formula(zut); ls(environment(zut)) confirms that the original 'formula's environment is unaffected
   predictor <- structure(formula,hyper_info=hyper_info) 
   class(predictor) <- c("predictor",class(formula))
   return(predictor)
@@ -938,7 +842,7 @@ as_precision <- function(corrMatrix) {
   return(vec_normIMRF)
 }
 
-.add_ZAfix_info <- function(AUGI0_ZX, ZAlist, sparse_precision, corr_types, processed) {
+.add_ZAfix_info <- function(AUGI0_ZX, ZAlist, sparse_precision, processed) {
   if ( ! any(AUGI0_ZX$vec_normIMRF)) {
     ZAfix <- .ad_hoc_cbind(ZAlist, as_matrix=FALSE)  
     if (sparse_precision) {
@@ -1033,7 +937,8 @@ as_precision <- function(corrMatrix) {
   }
   # add easily testable family name
   processed <- list2env(list(data=data, family=family, For=For,
-                             envir=list2env(list(),parent=environment(HLfit)))) 
+                             envir=list2env(list(), # bin_mu_tol=.spaMM.data$options$bin_mu_tol),
+                                            parent=environment(HLfit)))) 
   processed$clik_fn <- .get_clik_fn(family)
   #
   stop.on.error <- control.HLfit$stop.on.error ##  
@@ -1089,7 +994,7 @@ as_precision <- function(corrMatrix) {
   if (is.null(max.iter)) max.iter <- 200L
   processed$max.iter <- max.iter  
   #
-  ## control of maxit.mean in inner loop i.e. in .solve_IRLS_as_...(), dependent on y (Leucadendron_hard.R was revelatory)
+  ## control of maxit.mean in inner loop i.e. in .solve_IRLS_as_...(), dependent on y (caseStudies/Jeanne/Leucadendron_hard.R was revelatory)
   iter_mean_dispFix <- control.HLfit$iter.mean.dispFix ## documented
   if (is.null(iter_mean_dispFix)) iter_mean_dispFix <- control.HLfit$max.iter.mean ## public control
   if (is.null(iter_mean_dispFix)) {
@@ -1162,7 +1067,9 @@ as_precision <- function(corrMatrix) {
     attr(ZAlist,"exp_ranef_types") <- exp_ranef_types ## expanded
     
   } else rand.families <- list() ## ## corrects the default [gaussian()] when nrand=0
-  .assign_canonLink_G_LMMbool(family, processed) ## assigns processed$canonicalLink, $LMMbool, $GLMMbool using $lcrandfamfam
+  #
+  processed$prior.weights <- .preprocess_pw(subs_p_weights=substitute(prior.weights), nobs, validdata)
+  .assign_canonLink_G_LMMbool(family, processed) ## assigns processed$canonicalLink, $LMMbool, $GLMMbool using $lcrandfamfam, $prior.weights
   #
   HLmethod <- .preprocess_HLmethod(HLmethod, family, processed$lcrandfamfam) ## not a member of the return object
   HL <- eval(parse(text=paste0("c",substr(HLmethod,3,100)))) ## extracts the (...) part into a vector
@@ -1189,11 +1096,11 @@ as_precision <- function(corrMatrix) {
                                           init.HLfit=init.HLfit,
                                           X.pv=X.pv) ## possibly writes $corr_info$G_diagnosis! .../...
       ## F I X M E: ZAL diagnosis uses elements that may be modified afterwards and before .choose_QRmethod() reuses $G_diagnosis
-    processed$sparsePrecisionBOOL <- sparse_precision
+    processed$is_spprec <- sparse_precision
     #
     ## post-processing of corr_info depending on sparse_precision
     .process_corr_info_spprec(corr_types=corr_types,corr_info=corr_info, For=processed$For,sparse_precision=sparse_precision)
-  } else processed$sparsePrecisionBOOL <- FALSE ## for .do_TRACE()
+  } else processed$is_spprec <- FALSE ## for .do_TRACE()
   processed$init_HLfit <- init.HLfit ## this can be modified by dhglm-specific code.
   ###   OFFSET
   off <- model.offset(HLframes$mf) ## look for offset from (ori)Formula 
@@ -1236,10 +1143,12 @@ as_precision <- function(corrMatrix) {
   #
   pforpv <- ncol(X.pv)
   #
-  if(family$family == "binomial" && processed$bin_all_or_none && pforpv) {
-    if (For=="is_separated") {
+  if (For=="is_separated") {
+    if (pforpv) {
       return(is_separated(X.pv, as.numeric(y),verbose=FALSE)) 
-    } else isSeparated <- is_separated(X.pv, as.numeric(y))
+    } else return(FALSE)
+  } else if (family$family == "binomial" && processed$bin_all_or_none) {
+    isSeparated <- (pforpv && is_separated(X.pv, as.numeric(y)))
   }
   if (HL[1L]=="SEM") processed$SEMargs <- .preprocess_SEMargs(BinomialDen, nobs, control.HLfit, y)
   if (HL[1L]==0L) {processed$p_v_obj <-"hlik"} else processed$p_v_obj <-"p_v" ## objective for beta(_v) estim only: != outer obj 
@@ -1334,7 +1243,9 @@ as_precision <- function(corrMatrix) {
                   ZAlist[[it]] <- ZAlist[[it]][,activelevels,drop=FALSE]
                   attr(ZAlist[[it]],"is_incid") <- is_incid 
                 } 
-              } else if ( ! is.null(uniqueGeo)) { ## user-provided uniqueGeo (no example anywhere! :-) )
+              }
+              # not "else" because Matern/Cauchy must be combined with this!
+              if ( ! is.null(uniqueGeo)) { ## user-provided uniqueGeo (no example anywhere! :-) )
                 if (is.list(uniqueGeo)) { ## spaMM3.0 extended syntax
                   geo_info[[it]]$uniqueGeo <- uniqueGeo[[it]] 
                 } else geo_info[[it]]$uniqueGeo <- uniqueGeo #
@@ -1372,8 +1283,8 @@ as_precision <- function(corrMatrix) {
     processed$ZAlist <- ZAlist 
     processed$hyper_info <- .preprocess_hyper(processed=processed) # uses$ZAlist and $predictor
     #
-    processed$QRmethod <- .choose_QRmethod(ZAlist, predictor, corr_info=corr_info) ## (fixme: rethink) typically sparse for large Matern model ?
-    # QRmethod may well be dense for adjacency and then ZAfix will be dense. 
+    processed$QRmethod <- .choose_QRmethod(ZAlist, predictor, corr_info=corr_info,
+                                           is_spprec=processed$is_spprec) 
     nrd <- cum_n_u_h[nrand+1L]
     if ( ! .eval_as_mat_arg(processed)) {
       AUGI0_ZX <- list2env( list(I=.trDiagonal(n=nrd), ## avoids repeated calls to as() through rbind2...
@@ -1389,7 +1300,7 @@ as_precision <- function(corrMatrix) {
                                parent=environment(.preprocess))
     #
     if (sparse_precision) AUGI0_ZX$envir$method <- .spaMM.data$options$spprec_method  
-    AUGI0_ZX <- .add_ZAfix_info(AUGI0_ZX, ZAlist, sparse_precision, corr_types, processed)
+    AUGI0_ZX <- .add_ZAfix_info(AUGI0_ZX, ZAlist, sparse_precision, processed)
     
     # processed info for u_h inference
     processed$u_h_info <- .eval_v_h_bounds(cum_n_u_h, rand.families) 
@@ -1416,19 +1327,6 @@ as_precision <- function(corrMatrix) {
   coef3needed <- any(attr(rand.families,"unique.psi_M")!=0) # psi_M happening to be 0 only for gaussian(identity) ranef
   processed$vecdisneeded <- c( coef12needed, coef12needed, coef3needed )
   #
-  if (is.null(subs_p_weights <- substitute(prior.weights))) {
-    prior.weights <- structure(rep(1L,nobs),unique=TRUE) ## <- 1L prevented by glm -> model.frame(... prior.weights)
-  } else if ( ! (inherits(subs_p_weights,"call") && subs_p_weights[[1L]] == "quote") )  {
-    prior.weights <- as.vector(stats::model.weights(validdata)) ## as.vector as in say lm() protects against array1d
-    if ( ! is.numeric(prior.weights)) 
-      stop("'weights' must be a numeric vector")
-    if (any(prior.weights < 0)) 
-      stop("negative weights not allowed")
-    attr(prior.weights,"unique") <- (length(unique(prior.weights))==1L) 
-    #attr(prior.weights,"only1") <- all(upw==1L)
-  } else attr(prior.weights,"unique") <- FALSE ## when 'prior.weights' is a quoted expression   
-  processed$prior.weights <- prior.weights
-  #
   ## algorithms (control of defaults remains in the HLfit code)
   betaFirst <- control.HLfit$betaFirst ##  
   if (is.null(betaFirst)) {
